#!/usr/bin/env python

"""
Program code and data representations.
"""

class Block:

    "A code block."

    def __init__(self):
        self.code = []
        self.location = None

    def __repr__(self):
        return "Block(%r, location=%r)" % (id(self), self.location)

    def as_raw(self, objtable, paramtable):
        for i, item in enumerate(self.code):
            if hasattr(item, "location"):
                item.location = location + i
        return self.code

class DataObject:

    "A representation of a raw program data object."

    def __init__(self, classcode, attrcode, codeaddr, name, size, funccode=None):
        self.classcode = classcode
        self.attrcode = attrcode
        self.codeaddr = codeaddr
        self.name = name
        self.size = size
        self.funccode = funccode

    def with_size(self, size):
        return DataObject(self.classcode, self.attrcode, self.codeaddr, self.name, size, self.funccode)

    def with_callable(self, codeaddr):
        return DataObject(self.classcode, self.attrcode, codeaddr, self.name, self.size, self.funccode)

    def __repr__(self):
        return "%r # %s" % (
            (self.classcode, self.attrcode, self.codeaddr, self.funccode, self.size), self.name
            )

class FragmentObject:

    "A representation of a list fragment, used by list instances."

    def __init__(self, occupied_size, allocated_size):
        self.occupied_size = occupied_size
        self.allocated_size = allocated_size

    def __repr__(self):
        return "%r" % ((self.occupied_size, self.allocated_size),)

class Context:

    """
    A representation of a context used in a program, providing the special
    context value types.
    """

    def __init__(self, truth_value, repr):
        self.truth_value = truth_value
        self.repr = repr

    def __repr__(self):
        return self.repr

    def __nonzero__(self):
        return self.truth_value

    __shortrepr__ = __repr__

# A representation of a context that is replaced upon certain assignment
# operations.

ReplaceableContext = Context(0, "Replace")

# A representation of a context that is employed by classes so that
# instantiators have enough slots in the invocation frame, yet does not cause
# testing of the context or adjustment of the frame.

PlaceholderContext = Context(0, "Placeholder")

# vim: tabstop=4 expandtab shiftwidth=4
